@@ -40,21 +40,15 @@
 #include "gdm-signal-handler.h"
 #include "gdm-log.h"
 
-#include "gsm-consolekit.h"
 #include "gsm-gconf.h"
 #include "gsm-util.h"
 #include "gsm-manager.h"
-#include "gsm-xsmp-server.h"
+#include "gsm-session-fill.h"
 #include "gsm-store.h"
-
-#define GSM_GCONF_DEFAULT_SESSION_KEY           "/desktop/gnome/session/default_session"
-#define GSM_GCONF_REQUIRED_COMPONENTS_DIRECTORY "/desktop/gnome/session/required_components"
-#define GSM_GCONF_REQUIRED_COMPONENTS_LIST_KEY  "/desktop/gnome/session/required_components_list"
+#include "gsm-xsmp-server.h"
 
 #define GSM_DBUS_NAME "org.gnome.SessionManager"
 
-#define IS_STRING_EMPTY(x) ((x)==NULL||(x)[0]=='\0')
-
 static gboolean failsafe = FALSE;
 static gboolean show_version = FALSE;
 static gboolean debug = FALSE;
@@ -163,161 +157,6 @@ acquire_name (void)
         return TRUE;
 }
 
-/* This doesn't contain the required components, so we need to always
- * call append_required_apps() after a call to append_default_apps(). */
-static void
-append_default_apps (GsmManager *manager,
-                     const char *default_session_key,
-                     char      **autostart_dirs)
-{
-        GSList      *default_apps;
-        GSList      *a;
-        GConfClient *client;
-
-        g_debug ("main: *** Adding default apps");
-
-        g_assert (default_session_key != NULL);
-        g_assert (autostart_dirs != NULL);
-
-        client = gconf_client_get_default ();
-        default_apps = gconf_client_get_list (client,
-                                              default_session_key,
-                                              GCONF_VALUE_STRING,
-                                              NULL);
-        g_object_unref (client);
-
-        for (a = default_apps; a; a = a->next) {
-                char *app_path;
-
-                if (IS_STRING_EMPTY ((char *)a->data)) {
-                        continue;
-                }
-
-                app_path = gsm_util_find_desktop_file_for_app_name (a->data, autostart_dirs);
-                if (app_path != NULL) {
-                        gsm_manager_add_autostart_app (manager, app_path, NULL);
-                        g_free (app_path);
-                }
-        }
-
-        g_slist_foreach (default_apps, (GFunc) g_free, NULL);
-        g_slist_free (default_apps);
-}
-
-static void
-append_required_apps (GsmManager *manager)
-{
-        GSList      *required_components;
-        GSList      *r;
-        GConfClient *client;
-
-        g_debug ("main: *** Adding required apps");
-
-        client = gconf_client_get_default ();
-        required_components = gconf_client_get_list (client,
-                                                     GSM_GCONF_REQUIRED_COMPONENTS_LIST_KEY,
-                                                     GCONF_VALUE_STRING,
-                                                     NULL);
-        if (required_components == NULL) {
-                g_warning ("No required applications specified");
-        }
-
-        for (r = required_components; r != NULL; r = r->next) {
-                char       *path;
-                char       *default_provider;
-                const char *component;
-
-                if (IS_STRING_EMPTY ((char *)r->data)) {
-                        continue;
-                }
-
-                component = r->data;
-
-                path = g_strdup_printf ("%s/%s",
-                                        GSM_GCONF_REQUIRED_COMPONENTS_DIRECTORY,
-                                        component);
-
-                default_provider = gconf_client_get_string (client, path, NULL);
-                g_debug ("main: %s looking for component: '%s'", path, default_provider);
-                if (default_provider != NULL) {
-                        char *app_path;
-
-                        app_path = gsm_util_find_desktop_file_for_app_name (default_provider, NULL);
-                        if (app_path != NULL) {
-                                gsm_manager_add_autostart_app (manager, app_path, component);
-                        } else {
-                                g_warning ("Unable to find provider '%s' of required component '%s'",
-                                           default_provider,
-                                           component);
-                        }
-                        g_free (app_path);
-                }
-
-                g_free (default_provider);
-                g_free (path);
-        }
-
-        g_debug ("main: *** Done adding required apps");
-
-        g_slist_foreach (required_components, (GFunc)g_free, NULL);
-        g_slist_free (required_components);
-
-        g_object_unref (client);
-}
-
-static void
-maybe_load_saved_session_apps (GsmManager *manager)
-{
-        GsmConsolekit *consolekit;
-        char *session_type;
-
-        consolekit = gsm_get_consolekit ();
-        session_type = gsm_consolekit_get_current_session_type (consolekit);
-
-        if (g_strcmp0 (session_type, GSM_CONSOLEKIT_SESSION_TYPE_LOGIN_WINDOW) != 0) {
-                gsm_manager_add_autostart_apps_from_dir (manager, gsm_util_get_saved_session_dir ());
-        }
-
-        g_object_unref (consolekit);
-        g_free (session_type);
-}
-
-static void
-load_standard_apps (GsmManager *manager,
-                    const char *default_session_key)
-{
-        char **autostart_dirs;
-        int    i;
-
-        autostart_dirs = gsm_util_get_autostart_dirs ();
-
-        if (!failsafe) {
-                maybe_load_saved_session_apps (manager);
-
-                for (i = 0; autostart_dirs[i]; i++) {
-                        gsm_manager_add_autostart_apps_from_dir (manager,
-                                                                 autostart_dirs[i]);
-                }
-        }
-
-        /* We do this at the end in case a saved session contains an
-         * application that already provides one of the components. */
-        append_default_apps (manager, default_session_key, autostart_dirs);
-        append_required_apps (manager);
-
-        g_strfreev (autostart_dirs);
-}
-
-static void
-load_override_apps (GsmManager *manager,
-                    char      **override_autostart_dirs)
-{
-        int i;
-        for (i = 0; override_autostart_dirs[i]; i++) {
-                gsm_manager_add_autostart_apps_from_dir (manager, override_autostart_dirs[i]);
-        }
-}
-
 static gboolean
 signal_cb (int      signo,
            gpointer data)
@@ -432,10 +271,10 @@ main (int argc, char **argv)
         GsmXsmpServer    *xsmp_server;
         GdmSignalHandler *signal_handler;
         static char     **override_autostart_dirs = NULL;
-        static char      *default_session_key = NULL;
+        static char      *session_name = NULL;
         static GOptionEntry entries[] = {
                 { "autostart", 'a', 0, G_OPTION_ARG_STRING_ARRAY, &override_autostart_dirs, N_("Override standard autostart directories"), NULL },
-                { "default-session-key", 0, 0, G_OPTION_ARG_STRING, &default_session_key, N_("GConf key used to lookup default session"), NULL },
+                { "session", 0, 0, G_OPTION_ARG_STRING, &session_name, N_("Session to use"), NULL },
                 { "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
                 { "failsafe", 'f', 0, G_OPTION_ARG_NONE, &failsafe, N_("Do not load user-specified applications"), NULL },
                 { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, N_("Version of this application"), NULL },
@@ -511,14 +350,13 @@ main (int argc, char **argv)
         gdm_signal_handler_add (signal_handler, SIGINT, signal_cb, manager);
         gdm_signal_handler_set_fatal_func (signal_handler, shutdown_cb, manager);
 
-        if (override_autostart_dirs != NULL) {
-                load_override_apps (manager, override_autostart_dirs);
-        } else {
-                if (! IS_STRING_EMPTY (default_session_key)) {
-                        load_standard_apps (manager, default_session_key);
-                } else {
-                        load_standard_apps (manager, GSM_GCONF_DEFAULT_SESSION_KEY);
-                }
+        if (IS_STRING_EMPTY (session_name))
+                session_name = _gsm_manager_get_default_session (manager);
+
+        if (!gsm_session_fill (manager,
+                               override_autostart_dirs,
+                               session_name)) {
+                gsm_util_init_error (TRUE, "Failed to load session \"%s\"", session_name ? session_name : "(null)");
         }
 
         gsm_xsmp_server_start (xsmp_server);
